
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_OID_GENERATION_STAT_H
#define PG_OID_GENERATION_STAT_H

class ostream;
class OLog;
class ILog;

// oid generation statistics

class OidGenStat {
	protected:
		typedef int Table[4];

	public:
		OidGenStat();

		void reset();

		// these two must be called in the listed order
		void recordNeed(bool rep, bool pub);
		void recordGen(bool rep, bool pub);

		void store(OLog &log) const;
		void load(ILog &log);

		OidGenStat &operator +=(const OidGenStat &s);

		ostream &print(ostream &os, const String &pfx) const;

	protected:
		inline int toIdx(bool rep, bool pub) const;

		void mergeTable(Table &a, const Table &b) const;
		void writeTable(OLog &log, const Table &t) const;
		void readTable(ILog &log, Table &t);
		void printTable(ostream &os, const String &pfx, const Table &t) const;

	protected:
		Table theNeed;
		Table theLack;
		Table theGen;

	private:                 /* cached need info */
		bool needRep;
		bool needPub;
		bool waitForGen;
};

inline
int OidGenStat::toIdx(bool rep, bool pub) const {
	return (rep ? 2 : 0) | (pub ? 1 : 0);
}


inline
ostream &operator <<(ostream &os, const OidGenStat &s) { return s.print(os, ""); }

inline
OLog &operator <<(OLog &log, const OidGenStat &s) { s.store(log); return log; }

inline
ILog &operator >>(ILog &log, OidGenStat &s) { s.load(log); return log; }

#endif
