
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include <limits.h>

#include "xstd/Map.h"
#include "xstd/Rnd.h"
#include "xstd/String.h"
#include "base/RndPermut.h"

static int *ThePermTable = 0;
static int ThePermTableCap = 0;


void RndPermutConfig(int setSize, int seed) {
	delete ThePermTable;

	if (setSize > 0) {
		ThePermTable = new int[setSize];
		ThePermTableCap = setSize;
		RndPermutSeed(seed);
	} else {
		ThePermTable = 0;
		ThePermTableCap = 0;
	}
}

inline
void RndPermutSeedSwap(int x, int y) {
	const int h = ThePermTable[x];
	ThePermTable[x] = ThePermTable[y];
	ThePermTable[y] = h;
}

// fill set with "good" values
void RndPermutSeed(int seed) {
	RndGen rng(seed);
	for (int j = 0; j < ThePermTableCap; ++j)
		ThePermTable[j] = rng.ltrial();
	for (int i = 0; i < ThePermTableCap; ++i)
		RndPermutSeedSwap(i, rng(0, ThePermTableCap));
}

inline
int PermItem(int i) {
	return i >= 0 ?
		ThePermTable[i % ThePermTableCap] :
		ThePermTable[(i + INT_MAX) % ThePermTableCap];
}

int RndPermut(int n, int m) {
	const int a = n / ThePermTableCap;
	const int b = n % ThePermTableCap;
	const int offset = PermItem(a) - PermItem(m);
	return PermItem(offset + b);
}

RndGen *GenRndGen(const String &group) {
	static Map<RndGen*> gens;

	RndGen *gen = 0;
	if (!gens.find(group, gen)) {
		gen = new RndGen(RndPermut(gens.count(), group.len()));
		gens.add(group, gen);
	}
	return gen;
}

/* initialization */

int RndPermutfInit::TheUseCount = 0;

void RndPermutfInit::init() {
	RndPermutConfig(1048573); // 82,025-th prime number! (1M=1048576)
}

void RndPermutfInit::clean() {
	RndPermutConfig(0);
}
