
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_RANDOM_PERMUTATION_H
#define PG_RANDOM_PERMUTATION_H

#include "xstd/LibInit.h"

// We often need to convert a number to some uncorrellated uniformly
// distributed random number. Using seed-and-trial approach with a r.n.g. 
// does not work well if input stream of numbers is sequential and long.
// The permutator is also useful for seeding a r.n.g.

// Note: current permutation allows for a given [small] number of 
// high quality random values. The size of the output set is a parameter.

// changes the permutation set; expensive; 
// can be called multiple times, any time
// using prime numbers for set size is recommended
extern void RndPermutConfig(int setSize, int seed = 1);

// set size remains unchanged, but contents changes
extern void RndPermutSeed(int seed);

// use second parameter if you need to map two numbers into one uniform var
extern int RndPermut(int n, int m = 0);

// constant "offsets" to use as a second parameter to RndPermut
// when the first parameter is the same for a set of calls
enum { rndNone = 0, rndContentSel, rndSrvSeed, 
	rndContentPfx, rndContentExt,
	rndBodyIter, rndInjTbdPos, rndInjOff, rndInjProb,
	rndHotSetPos
};

// generate one seeded r.n.g. per group
class RndGen;
class String;
extern RndGen *GenRndGen(const String &group);

LIB_INITIALIZER(RndPermutfInit)

#endif
