
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_CLIENT_CONFIGURATION_H
#define PG_CLIENT_CONFIGURATION_H

class RndDistr;
class PopModel;
class RobotSym;
class WarmupPlan;

// Client configuration items that can be shared among multiple clients
class CltCfg {
	public:
		CltCfg();
		void configure(const RobotSym *aRobot);

		int originCount() const { return theOrigins.count(); }

		void startWarmup();
		void stopWarmup();

		int selectViserv();

	protected:
		void configureRTypes();
		void configureOrigins();

		void addOrigName(const NetAddr &oname);
		void cAddServers(int viserv);

	public:
		const RobotSym *theRobot;  // used to identify/share configs

		Array<int> theOrigins;     // idx to viserv
		RndDistr *theOriginSel;    // selects origin hosts
		RndDistr *theReqTypeSel;   // selects request types
		PopModel *thePopModel;     // popularity model to select old oids

		WarmupPlan *theWarmupPlan; // shared among similar configs

		NetAddr thePeerHttp;       // caching peer talking HTTP
		NetAddr thePeerIcp;        // caching peer talking ICP
		double thePublicRatio;     // how often to request a "public" object
		double theRecurRatio;      // how often to re-visit an old object
		double theEmbedRecurRatio; // how often to re-visit an old embedded obj

		Time theLaunchWin;
		int theWaitXactLmt;        // limit on the number of postponed requests
		int theIcpPort;            // IPC client should use this port

		bool genUniqUrls;          // each generated URL must be unique
		bool didWarmup;            // warmup plan was completed
};


class CltSharedCfgs: protected Array<CltCfg*> {
	public:
		CltCfg *getConfig(const RobotSym *cfg);

	protected:
		CltCfg *addConfig(const RobotSym *cfg);
};

#endif
