
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/h/iostream.h"
#include "xstd/h/iomanip.h"

#include "xstd/NetAddr.h"
#include "base/RndPermut.h"
#include "base/ObjId.h"
#include "runtime/HostMap.h"
#include "runtime/Viserv.h"
#include "csm/ContentCfg.h"
#include "csm/ContentMgr.h"
#include "csm/ContentSel.h"
#include "csm/oid2Url.h"


void Oid2UrlPath(const ObjId &oid, ostream &os) {
	const ContentCfg *ccfg = 0;
	if (oid.type() > 0) {
		ccfg = TheContentMgr.get(oid.type());
		Assert(ccfg);
	}

	if (ccfg) {
		const int seed = RndPermut(oid.hash(), rndContentPfx);
		if (const String &pfx = ccfg->url_pfx(seed))
			os << pfx;
	}

	os << 'w' << oid.world() << '/';

	// use non-default formatting to save space and reduce length variation
	{
		const ios_fmtflags flags = os.flags();
		const char fill = os.fill('0');
		os << hex << setfill('0');
		os << 't' << setw(2) << oid.type() << '/';
		os << '_' << setw(8) << oid.name();
		os.fill(fill);
		os.flags(flags);
	}

	if (ccfg) {
		const int seed = RndPermut(oid.hash(), rndContentExt);
		if (const String &ext = ccfg->url_ext(seed))
			os << ext;
	}
}

void Oid2Url(const ObjId &oid, ostream &os) {
	os << "http://";
	if (oid.viserv() >= 0) {
		const NetAddr &name = TheViservs.nameAt(oid.viserv());
		if (name.port() == 80)
			os << name.addrA();
		else
			os << name;
	} else {
		os << oid.viserv();
	}
	os << '/';

	Oid2UrlPath(oid, os);
}

int Oid2ContType(const ObjId &oid) {
	const HostCfg *hcfg = TheHostMap->at(oid.target());
	Assert(hcfg);
	Assert(hcfg->theContent);
	const ContentCfg *ccfg = hcfg->theContent->getDir(oid);
	return ccfg->id();
}
