
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PGL_NET_ADDRESS_BLOCK_H
#define PGL_NET_ADDRESS_BLOCK

class ostream;

// represents one "block" of an address [range]
class PglNetAddrBlock {
	public:
		virtual ~PglNetAddrBlock() {}

		virtual bool isA(const String &type) const = 0;
		virtual int count() const = 0;
		virtual PglNetAddrBlock *clone() const = 0;

		int diffCount(const PglNetAddrBlock &b) const;
		void merge(PglNetAddrBlock &b);

		virtual bool atLast() const = 0;

		virtual void start() = 0;
		virtual void next() = 0;

		virtual void print(ostream &os) const = 0;
		virtual void printCur(ostream &os) const = 0;
};

class PglStrAddrBlock: public PglNetAddrBlock {
	public:
		PglStrAddrBlock(const char *aStart, const char *aStop);

		virtual bool isA(const String &type) const;
		virtual int count() const;
		virtual PglNetAddrBlock *clone() const;

		int countDiffs(const PglStrAddrBlock &b) const;
		void mergeWith(const PglStrAddrBlock &b);

		virtual bool atLast() const;

		virtual void start();
		virtual void next();

		virtual void print(ostream &os) const;
		virtual void printCur(ostream &os) const;

	protected:
		const char *theStart;
		const char *theStop;
};

class PglRangeAddrBlock: public PglNetAddrBlock {
	public:
		PglRangeAddrBlock(int aStart, int aStop);

		virtual bool isA(const String &type) const;
		virtual int count() const;
		virtual PglNetAddrBlock *clone() const;

		int countDiffs(const PglRangeAddrBlock &b) const;
		void mergeWith(const PglRangeAddrBlock &b);

		virtual bool atLast() const;

		virtual void start();
		virtual void next();

		virtual void print(ostream &os) const;
		virtual void printCur(ostream &os) const;

	protected:
		bool contains(int pos) const;

	protected:
		int theStart;
		int theStop;

		int thePos;
};

#endif
