
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PGL_NET_ADDRESS_RANGE_H
#define PGL_NET_ADDRESS_RANGE_H

#include "xstd/Array.h"

class String;
class NetAddr;
class ArraySym;
class PglNetAddrBlock;


class PglNetAddrRange {
	public:
		typedef void (*AddrIter)(const NetAddr &);

	public:
		PglNetAddrRange();
		~PglNetAddrRange();

		void reset();

		// use subnet specs or guess an approprite netmask for the range
		void netmask(struct in_addr &pin) const;

		int addrCount() const;

		String toStr() const;
		ArraySym *toSyms() const;
		void toStrs(Array<String*> &strs) const;
		void toAddrs(Array<NetAddr*> &addrs) const;
		void toAddrs(AddrIter iter) const;

		bool parse(const String &val);

		// used by the lexer only
		void addStrBlock(const char *start, const char *stop);
		void addRangeBlock(int start, int stop);

		bool canMerge(const PglNetAddrRange &r) const;
		void merge(const PglNetAddrRange &r); // aborts if cannot
		
	protected:
		void startIter() const;
		bool nextIter() const;
		bool nextIter(int level) const;
		void currentIterAddr(String &str) const;
		int rangeCnt() const; // number of blocks specified using a range
		
	protected:
		Array<PglNetAddrBlock*> theBlocks; // address parts
		String theAddr;     // original image
		String theIfName;   // interface name
		int theSubnet;      // optional subnet specs
};

#endif
