
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "pgl/ContentSym.h"
#include "pgl/PglIntSym.h"
#include "pgl/PglTimeSym.h"
#include "pgl/PglRec.h"
#include "pgl/PglArraySym.h"
#include "pgl/PglStringSym.h"
#include "pgl/ServerSym.h"


ServerSym::ServerSym(): AgentSym("Server") {
	theRec->bAdd("int", "accept_lmt", 0);
	theRec->bAdd("Content[]", "contents", 0);
	theRec->bAdd("Content[]", "direct_access", 0);
	theRec->bAdd("string[]", "rep_types", 0);
}

ServerSym::ServerSym(const String aType, PglRec *aRec):	AgentSym(aType, aRec) {
}

bool ServerSym::isA(const String &type) const {
	return AgentSym::isA(type) || type == "Server";
}

SynSym *ServerSym::dupe(const String &type) const {
	if (isA(type))
		return new ServerSym(this->type(), theRec->clone());
	return AgentSym::dupe(type);
}

bool ServerSym::acceptLmt(int &lmt) const {
	return getInt("accept_lmt", lmt);
}

bool ServerSym::contents(Array<ContentSym*> &ccfgs, Array<double> &cprobs) const {
	SynSymTblItem *conti = 0;
	Assert(theRec->find("contents", conti));
	if (!conti->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)conti->sym()->cast("Array");
	a.copyProbs(cprobs);
	ArraySymExportM(ContentSym, a, "Content", ccfgs);
	return true;
}

bool ServerSym::directAccess(Array<ContentSym*> &dacfgs) const {
	SynSymTblItem *dai = 0;
	Assert(theRec->find("direct_access", dai));

	// use "contents" array if "direct_access" is not defined
	if (!dai->sym())
		Assert(theRec->find("contents", dai));
	if (!dai->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)dai->sym()->cast("Array");
	ArraySymExportM(ContentSym, a, "Content", dacfgs);
	return true;
}

RndDistr *ServerSym::repTypes(const Array<String*> &typeNames) const {
	return namesToDistr("rep_types", typeNames);
}
