
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/Select.h"
#include "xstd/Poll.h"
#include "runtime/SharedOpts.h"
#include "runtime/globals.h"

SharedOpts TheOpts;

SharedOpts::SharedOpts():
	theHelpOpt(this,      "help",               "list of options"),
	theVersOpt(this,      "version",            "package version info"),
	theHostTypeOpt(this,  "host_type",          "package build environment info"),

	theVerbLevel(this,    "verb_lvl <int>",     "console verbosity level", 0),
	theDumpFlags(this,    "dump <list>",        "dump request/replies/errors headers/bodies to console"),
	theDumpSize(this,     "dump_size <size>",   "limit size of individual msg dump", Size::KB(1)),
	theNotifAddr(this,    "notify <addr>",      "send notifications to a [remote] monitoring program", NetAddr()),
	theRunLabel(this,     "label <str>",        "run label"),

	theFDLimit(this,      "fd_limit <int>",     "decrease default fd limit"),

	theCfgFileName(this,  "config <file>",		"configuration file name"),
	theCfgDirs(this,      "cfg_dirs <list>",    "directories to search for cfg files"),

	theConsFileName(this, "console <file>",     "redirect console output", "-"),
	theLogFileName(this,  "log <file>",         "log file for samples"),
	theLogBufSize(this,   "log_buf_size <size>","buffer size for binary log"),
	theSmplLogFileName(this,"sample_log <file>",  "detailed binary log file"),
	theSmplLogBufSize(this, "sample_log_buf_size <size>","buffer size for sample log"),
	theStatCycleLen(this, "stats_cycle <time>", "statistics cycle length", Time::Sec(5)),
	doSyncPhases(this,    "sync_phases <bool>", "synchronize phases among remote processes", true),

	theFileScanner(this,  "file_scan <poll|select>", "system call to use for scanning ready files", new PG_PREFFERED_FILE_SCANNER),
	thePrioritySched(this,"priority_sched <int>", "priority level for urgent socket operations", 5),

	theFakeHosts(this,    "fake_hosts <list>",  "use given addresses, do not lookup real interfaces"),

	theIdleTout(this,     "idle_tout <time>",   "exit after inactivity timeout"),

	theRngSeed(this,      "rng_seed <int>",     "general purpose r.n.g. seed", 1),
	useUniqueWorld(this,  "unique_world <bool>","use URL set that is unique across runs", true)
{
}

bool SharedOpts::validate() const {
	if (theRngSeed <= 0)
		cerr << ThePrgName << ": r.n.g. seed must be positive; got: " << theRngSeed << endl;
	else
	if (theStatCycleLen <= 0)
		cerr << ThePrgName << ": the stats cycle length must be positive; got: " << theStatCycleLen << endl;
	else
	if (!theFileScanner)
		cerr << ThePrgName << ": the file scanner (e.g. `select' or `poll') must be specified" << endl;
	else
	if (thePrioritySched <= 0)
		cerr << ThePrgName << ": priority scheduling level must be positive; got: " << thePrioritySched << endl;
	else
	if (!theCfgFileName)
		cerr << ThePrgName << ": configuration file must be specified (see --config option)" << endl;
	else
		return OptGrp::validate();
	return false;
}
