
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_SERVER_CLASS_H
#define PG_SERVER_CLASS_H

#include "xstd/Socket.h"
#include "xstd/FileScanner.h"
#include "runtime/Agent.h"
#include "runtime/XactFarm.h"
#include "runtime/BcastRcver.h"
#include "server/SrvXact.h"
#include "server/SrvConnMgr.h"


class RndDistr;
class OidRec;
class PopModel;
class ServerSym;
class ContentSel;

class Server: public Agent, public FileScanner::User, public SrvConnMgr::User, public BcastRcver {
	public:
		static void LogState(OLog &log);
		static void Farm(XactFarm<SrvXact> *aFarm);

	public:
		Server();
		~Server();

		void configure(const ServerSym *cfg, const NetAddr &aHost);

		void start();

		// xactions need access to this
		int hostIdx() const { return theHostIdx; }
		PopModel *popModel() { return thePopModel; }

		void selectRepType(ObjId &oid);

		void noteXactDone(SrvXact *x);

		virtual void noteReadReady(int fd);
		virtual void noteConnReady(Connection *conn);

		virtual int logCat() const;

	protected:
		void configureContents(const ServerSym *cfg);
		void startXact(Connection *conn);
		void deaf();

		virtual void noteLogEvent(BcastChannel *ch, OLog &log);

	protected:
		static XactFarm<SrvXact> *TheXacts;

		SrvConnMgr *theConnMgr;
		Socket theSock;

		// XXX: these two should be moved to shared "SrvCfg" area
		PopModel *thePopModel;    // popularity model to repeat oids
		RndDistr *theRepTypeSel;  // selects reply types	

		int theHostIdx;           // index into the HostMap
		int theReqCount;          // number of accepted requests

		FileScanReserv theReserv;
};

#endif
