
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_SERVER_TRANSACTION_H
#define PG_SERVER_TRANSACTION_H

#include "xstd/AlarmClock.h"
#include "base/ObjTimes.h"
#include "runtime/Xaction.h"
#include "runtime/httpHdrs.h"

class Server;
class CltGroupRep;
class ContentCfg;
class BodyIter;
class WrBuf;
class ObjWorld;


class SrvXact: public Xaction, public AlarmUser {
	public:
		SrvXact() { SrvXact::reset(); }

		virtual void reset();

		void exec(Server *anOwner, Connection *aConn, Time delay);

		virtual void noteReadReady(int fd);
		virtual void noteWriteReady(int fd);
		virtual void wakeUp(const Alarm &a);

	protected:
		virtual Agent *owner();

		void makeRep(WrBuf &buf);

		bool shouldMake302Found() const;
		bool canMake302Found(ObjId &oid) const;
		bool make302Found(ostream &os);

		bool shouldMake304NotMod() const;
		void make304NotMod(ostream &os);

		void make200OK(ostream &os);

		void doStart();
		void noteDataReady();
		void interpretHeader();

		Size calcRepSize() const;
		bool calcCachability() const;
		bool setViserv(const NetAddr &name);
		void updatePubWorld(const ObjWorld &newSlice);
		void putRemWorld(ostream &os, const ObjWorld &oldSlice);
		virtual Size put200OkBody(ostream &os);
		void putStdFields(ostream &os) const;

		virtual bool setTarget(const NetAddr &target);
		virtual void finish(Error err);
		virtual void logStats(OLog &ol) const;

	protected:
		Server *theOwner;
		CltGroupRep *theCltGroupRep;
		ContentCfg *theContentCfg;
		BodyIter *theBodyIter;
		ReqHdr theReqHdr;

		ObjTimes theTimes;    // mod, lmt, and such
		Size theRepHdrSize;

		Size theProducedSize;

	private:
		int theSeed;          // oid-based seed used for r.n.g
};

#endif

