
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "xstd/xstd.h"

#include <ctype.h>
#include "xstd/h/netinet.h"
#include "xstd/h/net/if.h"
#include "xstd/h/iostream.h"
#include "xstd/h/iomanip.h"

#include "xstd/getIfAddrs.h"
#include "xstd/NetIface.h"


bool NetIface::GetAddrs(Array<in_addr> &addrs, const char *ifname) {
	return GetIfAddrs(addrs, ifname);
}

NetIface::NetIface() {
	Must(theSock.create(AF_INET, SOCK_DGRAM, 0));
}

NetIface::~NetIface() {
	if (theSock)
		theSock.create(AF_INET, SOCK_DGRAM, 0);
}

void NetIface::name(const String &aName) {
	theName = aName;
}

NetAddr NetIface::primaryAddr() const {
	Assert(theName);
	sockaddr_in addr;
	if (!theSock.getIfAddr(theName, (sockaddr&)addr))
		cerr << "cannot get primary address of the `" << theName << "' interface" << endl << xexit;
	NetAddr pa(addr);
	pa.port(-1);
	return pa;
}

void NetIface::getAliases(Array<in_addr> &olds) const {
	Assert(theName);
	Must(GetAddrs(olds, theName));

	// remove primary address from the aliases array
	const NetAddr pa = primaryAddr();
	int paIdx = -1;
	for (int i = 0; paIdx < 0 && i < olds.count(); ++i) {
		if (olds[i] == pa.addrN())
			paIdx = i;
	}
	Must(paIdx >= 0);
	if (paIdx < olds.count()-1)
		olds.memmove(paIdx, olds.items()+paIdx+1, olds.count()-paIdx-1);
	olds.pop();
}

int NetIface::delAliases() {
	Assert(theName);
	Array<in_addr> olds;
	getAliases(olds);

	// reverse order seems to be important on Linux where
	// one cannot delete :1 after deleting :0
	for (int i = olds.count() - 1; i >= 0; --i) {
		InetIfAliasReq r(theName);
		r.addr().sin_addr = olds[i];
		Must(theSock.delIfAddr(r, i));
	}
	return olds.count();
}

bool NetIface::addAlias(const in_addr &addr, int idx, in_addr netmask) {
	Assert(theName);
	InetIfAliasReq r(theName);
	r.addr().sin_addr = addr;

	// set network mask (geting old netmask is not documented?!)
	r.mask() = r.addr();
	r.mask().sin_addr = netmask;

	// set broacast address
	r.broad() = r.addr();
	r.broad().sin_addr.s_addr |= ~(unsigned int)r.mask().sin_addr.s_addr;

	return theSock.addIfAddr(r, idx) || !Error::LastExcept(EEXIST);
}

bool NetIface::addAliases(const Array<in_addr> &aliases, in_addr netmask) {
	for (int i = 0; i < aliases.count(); ++i) {
		if (!addAlias(aliases[i], i, netmask))
			return false;
	}
	return true;
}
